//+------------------------------------------------------------------+
//|                                                   T3 crosses.mq4 |
//|                                                           mladen |
//|                                                                  |
//|                                                                  |
//| original T3 developed by Tim Tilson (TASC January 1998)          |
//| period lag modification by Bob Fulks and Alex Matulich 4/2003    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 Red
#property indicator_color2 Maroon
#property indicator_color3 LimeGreen
#property indicator_color4 Green
#property indicator_color5 Gold
#property indicator_color6 Green
#property indicator_width1 2
#property indicator_width2 1
#property indicator_width3 2
#property indicator_width4 1
#property indicator_width5 2
#property indicator_width6 2

//
//
//
//
//

extern double CyclePeriodAlpha = 0.07;
extern double T3FastPeriod     = 0.5;
extern int    T3FastPrice      = PRICE_CLOSE;
extern double T3FastHot        = 1.2;
extern bool   T3FastOriginal   = false;
extern bool   T3FastAdaptive   = true;
extern double T3SlowPeriod     = 1.0;
extern int    T3SlowPrice      = PRICE_CLOSE;
extern double T3SlowHot        = 1.2;
extern bool   T3SlowOriginal   = false;
extern bool   T3SlowAdaptive   = true;

//
//
//
//
//

double t3gCross[];
double t3rCross[];
double upBuffer1[];
double upBuffer2[];
double dnBuffer1[];
double dnBuffer2[];
double t3diff[];
double trend[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(8);
      SetIndexBuffer(0,dnBuffer1); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,dnBuffer2); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,upBuffer1); SetIndexStyle(2,DRAW_HISTOGRAM);
      SetIndexBuffer(3,upBuffer2); SetIndexStyle(3,DRAW_HISTOGRAM);
      SetIndexBuffer(4,t3rCross);
      SetIndexBuffer(5,t3gCross);
      SetIndexBuffer(6,t3diff);  
      SetIndexBuffer(7,trend);
      
      //
      //
      //
      //
      //
   
      T3FastPeriod = MathMax(0.01,T3FastPeriod);
      T3SlowPeriod = MathMax(0.01,T3SlowPeriod);

   IndicatorShortName("T3 trix ("+DoubleToStr(T3FastPeriod,2)+", "+DoubleToStr(T3FastHot,2)+"; "+DoubleToStr(T3SlowPeriod,2)+", "+DoubleToStr(T3SlowHot,2)+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double work[][2];

int start()
{
   int i,r,limit,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (ArrayRange(work,0) != Bars) ArrayResize(work,Bars);

   //
   //
   //
   //
   //

      for(i=limit, r=Bars-i-1; i>=0; i--,r++)
      {
         double period1 = iCustom(NULL,0,"CyclePeriod_v2",T3FastPrice,CyclePeriodAlpha,0,i);
         double period2 = iCustom(NULL,0,"CyclePeriod_v2",T3SlowPrice,CyclePeriodAlpha,0,i);
         work[r][0]   = iT3a(iMA(NULL,0,1,0,MODE_SMA,T3FastPrice,i),period1*T3FastPeriod,T3FastHot,T3FastOriginal,T3FastAdaptive,i,0);  
         work[r][1]   = iT3a(iMA(NULL,0,1,0,MODE_SMA,T3SlowPrice,i),period2*T3SlowPeriod,T3SlowHot,T3SlowOriginal,T3SlowAdaptive,i,1);
         t3gCross[i]  = work[r][0]-work[r-1][0];
         t3rCross[i]  = work[r][1]-work[r-1][1];
         t3diff[i]    = t3gCross[i]-t3rCross[i];
         upBuffer1[i] = EMPTY_VALUE;
         upBuffer2[i] = EMPTY_VALUE;
         dnBuffer1[i] = EMPTY_VALUE;
         dnBuffer2[i] = EMPTY_VALUE;
         trend[i]     = trend[i+1];
            if (t3diff[i]>t3diff[i+1]) trend[i] =  1;
            if (t3diff[i]<t3diff[i+1]) trend[i] = -1;
            if (t3diff[i]>0)
               if (trend[i] == 1)
                     upBuffer1[i] = t3diff[i];
               else  upBuffer2[i] = t3diff[i];
            else               
               if (trend[i] == 1)
                     dnBuffer2[i] = t3diff[i];
               else  dnBuffer1[i] = t3diff[i];
      }
      return(0);
}

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
//
//
//

double workT3[][14];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5
#define _atr    6

//
//
//
//
//

double iT3a(double price, double period, double hot, bool original, bool adaptive, int i, int forInstance=0)
{
   int r      = Bars-i-1;
   int buffer = forInstance*7;

   //
   //
   //
   //
   //
   
      if (ArrayRange(workT3,0) !=Bars)                  ArrayResize(workT3,Bars);
      if (ArrayRange(workT3Coeffs,0) < (forInstance+1)) ArrayResize(workT3Coeffs,forInstance+1);
      if (adaptive)
      {
         workT3[r][_atr+buffer] = iATR(NULL,0,period,i);
            double max = workT3[r][_atr+buffer];
            double min = workT3[r][_atr+buffer];
            for (int k=1; k<period && (r-k>=0); k++)
               {
                  max = MathMax(max,workT3[r-k][_atr+buffer]);
                  min = MathMin(min,workT3[r-k][_atr+buffer]);
               }
            if (min!=max)
                  double atn = 1.0-(workT3[r][_atr+buffer]-min)/(max-min);
            else         atn = 0.5;
            period = period*(atn+1.0)/2.0;
      }
   
   //
   //
   //
   //
   //
   
      if (workT3Coeffs[forInstance][_period] != period)
      {
        workT3Coeffs[forInstance][_period] = period;
           double a = hot;
               workT3Coeffs[forInstance][_c1] = -a*a*a;
               workT3Coeffs[forInstance][_c2] = 3*a*a+3*a*a*a;
               workT3Coeffs[forInstance][_c3] = -6*a*a-3*a-3*a*a*a;
               workT3Coeffs[forInstance][_c4] = 1+3*a+a*a*a+3*a*a;
               if (original)
                    workT3Coeffs[forInstance][_alpha] = 2.0/(1.0 + period);
               else workT3Coeffs[forInstance][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
      }
   
   //
   //
   //
   //
   //
   
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[forInstance][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[forInstance][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[forInstance][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[forInstance][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[forInstance][_c4]*workT3[r][2+buffer]);
}